@@ -24,9 +24,12 @@
 
 package Init::Services;
 
+my $SERVICE_START = 0;
+my $SERVICE_STOP  = 1;
+
 use Init::ServicesList;
 
-sub get_runlevel_roles
+sub get_runlevels
 {
   my (%dist_map, %runlevels);
   my ($desc, $distro);
@@ -41,40 +44,20 @@ sub get_runlevel_roles
      "redhat-8.0"       => "redhat-6.2",
      "mandrake-9.0"     => "redhat-6.2",
      "conectiva-9"      => "redhat-6.2",
-     "debian-3.0"       => "debian-3.0",
+     "debian-3.0"       => "redhat-6.2",
      "suse-9.0"         => "redhat-6.2",
      "pld-1.0"          => "redhat-6.2",
      "vine-3.0"         => "redhat-6.2",
      "slackware-9.1.0"  => "slackware-9.1.0",
      "gentoo"           => "gentoo",
-     "freebsd-5"        => "freebsd-5",
+     "freebsd-5"        => "slackware-9.1.0",
     );
 
   %runlevels=
     (
-     "redhat-6.2"      => [["0",         "HALT"      ],
-                           ["1",         "RECOVER"   ],
-                           ["2",         "NONE"      ],
-                           ["3",         "TEXT"      ],
-                           ["4",         "NONE"      ],
-                           ["5",         "GRAPHICAL" ],
-                           ["6",         "REBOOT"    ]],
-
-     "debian-3.0"      => [["0",         "HALT"      ],
-                           ["1",         "RECOVER"   ],
-                           ["2",         "NONE"      ],
-                           ["3",         "NONE"      ],
-                           ["4",         "NONE"      ],
-                           ["5",         "NONE"      ],
-                           ["6",         "REBOOT"    ]],
-
-     "gentoo"          => [["boot",      "BOOT"      ],
-                           ["default",   "GRAPHICAL" ],
-                           ["nonetwork", "RECOVER"   ]],
-
-     "freebsd-5"       => [["default",   "GRAPHICAL" ]],
-
-     "slackware-9.1.0" => [["default",   "GRAPHICAL" ]]
+     "redhat-6.2"      => [ "0", "1", "2", "3", "4", "5", "6" ],
+     "gentoo"          => [ "boot", "default", "nonetwork" ],
+     "freebsd-5"       => [ "default" ],
     );
 
   $distro = $dist_map{$Utils::Backend::tool{"platform"}};
@@ -155,11 +138,11 @@ sub get_sysv_runlevels_status
 
     if ($action eq "S")
 		{
-      push @arr, [ $runlevel, "start", $priority ];
+      push @arr, [ $runlevel, $SERVICE_START, $priority ];
     }
 		elsif ($action eq "K")
 		{
-      push @arr, [ $runlevel, "stop", $priority ];
+      push @arr, [ $runlevel, $SERVICE_STOP, $priority ];
 		}
 	}
 	
@@ -179,14 +162,9 @@ sub get_sysv_service_info
 	return unless (-x $service);
 
 	$script = &get_sysv_service_name ($service);
-		
-	# We have to check out if the service is in the "forbidden" list
-	return if (&Init::ServicesList::is_forbidden ($script));
-
 	$runlevels = &get_sysv_runlevels_status($script);
-  $role = &Init::ServicesList::get_role ($script);
 
-  return ($script, $role, $runlevels);
+  return ($script, $runlevels);
 }
 
 # This function gets an ordered array of the available services from a SysV system
@@ -236,21 +214,47 @@ sub add_sysv_link
   &Utils::Report::leave ();
 }
 
+sub run_sysv_initd_script
+{
+  my ($service, $arg) = @_;
+  my ($rc_path, $initd_path);
+  my $str;
+
+  &Utils::Report::enter ();
+  
+  ($rcd_path, $initd_path) = &get_sysv_paths ();
+
+  if (-f "$initd_path/$service")
+  {
+    if (&Utils::File::run ("$initd_path/$service $arg") == 0)
+    {
+      &Utils::Report::do_report ("service_sysv_op_success", $service, $arg);
+      &Utils::Report::leave ();
+      return 0;
+    }
+  }
+  
+  &Utils::Report::do_report ("service_sysv_op_failed", $service, $arg);
+  &Utils::Report::leave ();
+  return -1;
+}
+
 sub set_sysv_service
 {
   my ($service) = @_;
-  my ($script, $priority, $runlevels);
+  my ($script, $priority, $runlevels, $default_runlevel);
   my ($runlevel, $action, %configured_runlevels);
 
   ($rcd_path, $initd_path, $relative_path) = &get_sysv_paths ();
 
   $script = $$service[0];
-  $runlevels = $$service[2];
+  $runlevels = $$service[1];
+  $default_runlevel = &get_sysv_default_runlevel ();
 
   foreach $r (@$runlevels)
   {
     $runlevel = $$r[0];
-    $action   = ($$r[1] eq "start") ? "S" : "K";
+    $action   = ($$r[1] == $SERVICE_START) ? "S" : "K";
     $priority = sprintf ("%0.2d", $$r[2]);
 
     $configured_runlevels{$runlevel} = 1;
@@ -259,6 +263,11 @@ sub set_sysv_service
     {
       &remove_sysv_link ($rcd_path, $runlevel, $script);
       &add_sysv_link ($rcd_path, $relative_path, $runlevel, $action, $priority, $script);
+
+      if ($runlevel eq $default_runlevel)
+      {
+        &run_sysv_initd_script ($script, ($$r[1] == $SERVICE_START) ? "start" : "stop");
+      }
     }
   }
 
@@ -271,6 +280,11 @@ sub set_sysv_service
     if (!exists $configured_runlevels{$runlevel})
     {
       &remove_sysv_link ($rcd_path, $runlevel, $script);
+
+      if ($runlevel eq $default_runlevel)
+      {
+        &run_sysv_initd_script ($script, "stop");
+      }
     }
   }
 }
@@ -299,7 +313,7 @@ sub get_filerc_runlevels_status
 
     foreach $runlevel (@runlevels)
     {
-      push @arr, [ $runlevel, "start", $priority ];
+      push @arr, [ $runlevel, $SERVICE_START, $priority ];
     }
   }
 
@@ -311,7 +325,7 @@ sub get_filerc_runlevels_status
 
     foreach $runlevel (@runlevels)
     {
-      push @arr, [ $runlevel, "stop", $priority ];
+      push @arr, [ $runlevel, $SERVICE_STOP, $priority ];
     }
   }
 
@@ -330,12 +344,9 @@ sub get_filerc_service_info
     $start_service = $3;
     $script = $4;
 
-    return if (&Init::ServicesList::is_forbidden ($script));
-
     $runlevels = &get_filerc_runlevels_status ($start_service, $stop_service, $priority);
-    $role = &Init::ServicesList::get_role ($script);
 
-    return ($script, $role, $runlevels);
+    return ($script, $runlevels);
   }
 
   return;
@@ -394,42 +405,57 @@ sub set_filerc_service
 {
   my ($buff, $initd_path, $service) = @_;
   my (%hash, $priority, $line, $str);
+  my ($script, $default_runlevel, %configured_runlevels);
 
-  $runlevels = $$service[2];
+  $script = $$service[0];
+  $runlevels = $$service[1];
+  $default_runlevel = &get_sysv_default_runlevel ();
 
   foreach $i (@$runlevels)
   {
     $priority = 0 + $$i[2];
     $priority = 50 if ($priority == 0); #very rough guess
+    $configured_runlevels {$$i[0]} = 1;
 
-    if ($$i[1] eq "start")
+    if ($$i[1] == $SERVICE_START)
     {
-      $hash{$priority}{"start"} = [] if (!$hash{$priority}{"start"});
-      push @{$hash{$priority}{"start"}}, $$i[0];
+      $hash{$priority}{$SERVICE_START} = [] if (!$hash{$priority}{$SERVICE_START});
+      push @{$hash{$priority}{$SERVICE_START}}, $$i[0];
     }
     else
     {
-      $hash{$priority}{"stop"} = [] if (!$hash{$priority}{"stop"});
-      push @{$hash{$priority}{"stop"}}, $$i[0];
+      $hash{$priority}{$SERVICE_STOP} = [] if (!$hash{$priority}{$SERVICE_STOP});
+      push @{$hash{$priority}{$SERVICE_STOP}}, $$i[0];
+    }
+
+    if ($$i[0] eq $default_runlevel)
+    {
+      &run_sysv_initd_script ($script, ($$i[1] == $SERVICE_START) ? "start" : "stop");
     }
   }
 
   foreach $priority (keys %hash)
   {
     $line  = sprintf ("%0.2d", $priority) . "\t";
-    $line .= &concat_filerc_runlevels (@{$hash{$priority}{"stop"}}) . "\t";
-    $line .= &concat_filerc_runlevels (@{$hash{$priority}{"start"}}) . "\t";
-    $line .= $initd_path . "/" . $$service{"script"} . "\n";
+    $line .= &concat_filerc_runlevels (@{$hash{$priority}{$SERVICE_STOP}}) . "\t";
+    $line .= &concat_filerc_runlevels (@{$hash{$priority}{$SERVICE_START}}) . "\t";
+    $line .= $initd_path . "/" . $script . "\n";
 
     push @$buff, $line;
   }
+
+  # stop the service if it's not configured
+  if (!$configured_runlevels {$default_runlevel})
+  {
+    &run_sysv_initd_script ($script, "stop");
+  }
 }
 
 sub set_filerc_services
 {
   my ($services) = @_;
   my ($buff, $lineno, $line, $file);
-  my ($rcd_path, $initd_path, $relative_path) = &sysv_get_paths ();
+  my ($rcd_path, $initd_path, $relative_path) = &get_sysv_paths ();
 
   $file = "/etc/runlevel.conf";
 
@@ -450,6 +476,7 @@ sub set_filerc_services
     $$buff[$i] =~ /.*\/etc\/init\.d\/(.*)/;
 
     # we need to keep the forbidden services and the services that only start in rcS.d
+    # FIXME: need to remove this call to is_forbidden
     if (!&Init::ServicesList::is_forbidden ($1))
     {
       delete $$buff[$i];
@@ -470,48 +497,10 @@ sub set_filerc_services
 }
 
 # this functions get a list of the services that run on a bsd init
-sub get_bsd_service_info
+sub get_bsd_scripts_list
 {
-  my ($service) = @_;
-  my ($script);
-  my (%hash);
-	my (@arr, @rl);
-
-  $script = $service;
-  $script =~ s/^.*\///;
-  $script =~ s/^rc\.//;
-
-  return undef if (! Utils::File::exists ($service));
-
-  return undef if (&Init::ServicesList::is_forbidden ($script));
-
-  $hash {"script"} = $service;
-
-  # we hardcode the fourth runlevel, it's the graphical one
-  if ( -x $service)
-  {
-    push @arr, { "name"   => 4,
-                 "action" => "start" };
-  }
-  else
-  {
-    push @arr, { "name"   => 4,
-                 "action" => "stop" };
-  }
-
-	push @rl, { "runlevel" => \@arr };
-  
-	$hash{"runlevels"} = \@rl;
-  $hash{"role"} = &Init::ServicesList::get_role ($script);
-  
-  return \%hash;
-}
-
-sub get_bsd_services
-{
-  my (%ret);
   my ($files) = [ "rc.M", "rc.inet2", "rc.4" ];
-  my ($file);
+  my ($file, $i, @scripts);
 
   foreach $i (@$files)
   {
@@ -529,44 +518,127 @@ sub get_bsd_services
 
       if ($line =~ /^if[ \t]+\[[ \t]+\-x[ \t]([0-9a-zA-Z\/\.\-_]+) .*\]/)
       {
-        my (%hash);
         $service = $1;
 
-        $hash = &get_bsd_service_info ($service);
-
-        if ($hash ne undef)
-        {
-          $ret{$service} = $hash;
-        }
+        push @scripts, $service;
       }
     }
 
     Utils::File::close_file ($fd);
   }
 
-  return \%ret;
+  return \@scripts;
+}
+
+sub get_bsd_service_info
+{
+  my ($service) = @_;
+  my ($script, @runlevels);
+
+  $script = $service;
+  $script =~ s/^.*\///;
+  $script =~ s/^rc\.//;
+
+  return undef if (! Utils::File::exists ($service));
+
+  # hardcode the fourth runlevel, it's the graphical one
+  if ( -x $service)
+  {
+    push @runlevels, [ "4", $SERVICE_START, 0 ];
+  }
+  else
+  {
+    push @runlevels, [ "4", $SERVICE_STOP, 0 ];
+  }
+
+  # FIXME: $service contains full path, would
+  # be nice to just show the $script name
+  return ($service, $runlevels);
+}
+
+sub get_bsd_services
+{
+  my (@arr, @scripts, $script);
+
+  $scripts = &get_bsd_scripts_list ();
+
+  foreach $script (@scripts)
+  {
+    my (@info);
+
+    @info = &get_bsd_service_info ($script);
+    push @arr, \@info if (scalar (@info));
+  }
+
+  return \@arr;
+}
+
+sub run_bsd_script
+{
+  my ($service, $arg) = @_;
+  my ($chmod) = 0;
+
+  return if (!&Utils::File::exists ($service));
+
+  # if it's not executable then chmod it
+  if (!((stat ($service))[2] & (S_IXUSR || S_IXGRP || S_IXOTH)))
+  {
+    $chmod = 1;
+    &Utils::File::run ("chmod ugo+x $service");
+  }
+  
+  &Utils::File::run ("$service $arg");
+
+  # return it to it's normal state
+  if ($chmod)
+  {
+    &Utils::File::run ("chmod ugo-x $service");
+  }
 }
 
 # This function stores the configuration in a bsd init
+sub get_bsd_service_path
+{
+  my ($script) = @_;
+  my (@scripts, $elem, $i);
+
+  $scripts = &get_bsd_scripts_list ();
+
+  foreach $i (@scripts)
+  {
+    $elem = $i;
+    $elem =~ s/^.*\///;
+    $elem =~ s/^rc\.//;
+
+    return $i if ($elem eq $script);
+  }
+
+  return undef;
+}
+
 sub set_bsd_services
 {
   my ($services) = @_;
-  my ($script, $runlevels);
+  my ($script, $runlevels, @scripts);
 
-	foreach $service (@$services)
-	{
-    $script = $$service[0];
-    $runlevels = $$service[2];
+  foreach $service (@$services)
+  {
+    $script = &get_bsd_service_path ($$service[0]);
+    $runlevels = $$service[1];
     $runlevel  = $$runlevels[0];
 
+    next if ($script eq undef);
+
     $action = $$runlevel[1];
 
-    if ($action eq "start")
+    if ($action == $SERVICE_START)
     {
       &Utils::File::run ("chmod ugo+x $script");
+      &run_bsd_script ($script, "start");
     }
     else
     {
+      &run_bsd_script ($script, "stop");
       &Utils::File::run ("chmod ugo-x $script");
     }
   }
@@ -688,11 +760,11 @@ sub get_gentoo_runlevel_status_by_service
   {
     if (defined $start_runlevels{$runlevel})
     {
-      push @arr, [ $runlevel, "start", 0 ];
+      push @arr, [ $runlevel, $SERVICE_START, 0 ];
     }
     else
     {
-      push @arr, [ $runlevel, "stop", 0 ];
+      push @arr, [ $runlevel, $SERVICE_STOP, 0 ];
     }
   }
 
@@ -705,13 +777,9 @@ sub get_gentoo_service_info
 	my ($script, @actions, @runlevels);
 	my ($role);
 	
-	# We have to check out if the service is in the "forbidden" list
-	return if (&Init::ServicesList::is_forbidden ($service));
-
   $runlevels = &get_gentoo_runlevel_status_by_service ($service);
-  $role = &Init::ServicesList::get_role ($service);
 
-  return ($service, $role, $runlevels);
+  return ($service, $runlevels);
 }
 
 sub get_gentoo_services
@@ -731,16 +799,40 @@ sub get_gentoo_services
   return \@arr;
 }
 
+#FIXME: almost equal to the sysv equivalent
+sub run_gentoo_script
+{
+  my ($service, $arg) = @_;
+
+  &Utils::Report::enter ();
+
+  if (&gentoo_service_exists ($service))
+  {
+    if (!&Utils::File::run ("/etc/init.d/$service $arg"))
+    {
+      &Utils::Report::do_report ("service_sysv_op_success", $service, $arg);
+      &Utils::Report::leave ();
+	    return 0;
+	  }
+  }
+
+  &Utils::Report::do_report ("service_sysv_op_failed", $service, $arg);
+  &Utils::Report::leave ();
+  return -1;
+}
+
 sub set_gentoo_service_status
 {
   my ($script, $rl, $action) = @_;
 
-  if ($action eq "start")
+  if ($action == $SERVICE_START)
   {
     &Utils::File::run ("rc-update add $script $rl");
+    &run_gentoo_script ($script, "start");
   }
-  elsif ($action eq "stop")
+  else
   {
+    &run_gentoo_script ($script, "stop");
     &Utils::File::run ("rc-update del $script $rl");
   }
 }
@@ -754,7 +846,7 @@ sub set_gentoo_services
   foreach $service (@$services)
   {
     $script = $$service[0];
-    $arr = $$service[2];
+    $arr = $$service[1];
 
     foreach $i (@$arr)
     {
@@ -798,50 +890,54 @@ sub get_rcng_status_by_service
 
 sub get_rcng_service_info
 {
-  my ($service) = @_;
-  my ($script, @actions, @runlevels);
-  my (%hash, @arr, @rl);
+  my ($script) = @_;
+  my (@runlevels);
 
-  # We have to check if the service is in the "forbidden" list
-  return undef if (&Init::ServicesList::is_forbidden ($service));
-
-  $hash{"script"} = $service;
-
-  if (get_rcng_status_by_service ($service))
+  if (get_rcng_status_by_service ($script))
   {
-    push @arr, { "name"   => "default",
-                 "action" => "start" };
+    push @runlevels, [ "default", $SERVICE_START, 0 ];
   }
   else
   {
-    push @arr, { "name"   => "default",
-                 "action" => "stop" };
+    push @runlevels, [ "default", $SERVICE_STOP, 0 ];
   }
 
-  push @rl,  { "runlevel", \@arr };
-
-  $hash {"runlevels"} = \@rl;
-  $hash {"role"} = &Init::ServicesList::get_role ($service);
-
-  return \%hash;
+  return ($script, $runlevels);
 }
 
 sub get_rcng_services
 {
   my ($service);
-  my (%ret);
+  my (@arr);
 
   foreach $service (<$gst_prefix/etc/rc.d/*>)
   {
-    my (%hash);
+    my (@info);
 
     $service =~ s/.*\///;
-    $hash = &get_rcng_service_info ($service);
-
-    $ret{$service} = $hash if ($hash ne undef);
+    @info = &get_rcng_service_info ($service);
+    push @arr, \@info if (scalar (@info));
   }
 
-  return \%ret;
+  return \@arr;
+}
+
+sub run_rcng_script
+{
+  my ($service, $arg) = @_;
+
+  &Utils::Report::enter ();
+
+  if (!&Utils::File::run ("/etc/rc.d/$service $arg"))
+  {
+    &Utils::File::do_report ("service_sysv_op_success", $service, $arg);
+    &Utils::File::leave ();
+    return 0;
+  }
+
+  &Utils::File::do_report ("service_sysv_op_failed", $service, $arg);
+  &Utils::File::leave ();
+  return -1;
 }
 
 # These functions store the configuration of a rcng init
@@ -873,6 +969,8 @@ sub set_rcng_service_status
         {
           &Utils::Replace::set_sh_bool ($rcconf, $key, "YES", "NO", $action);
         }
+
+        &run_rcng_script ($service, ($action) ? "forcestart" : "forcestop");
       }
     }
 
@@ -882,11 +980,13 @@ sub set_rcng_service_status
   {
     if ($action)
     {
-      Utils::File::copy_file ("/usr/local/etc/rc.d/$service.sh.sample",
+      &Utils::File::copy_file ("/usr/local/etc/rc.d/$service.sh.sample",
                               "/usr/local/etc/rc.d/$service.sh");
+      &run_rcng_script ($service, "forcestart");
     }
     else
     {
+      &run_rcng_script ($service, "forcestop");
       Utils::File::remove ("/usr/local/etc/rc.d/$service.sh");
     }
   }
@@ -910,6 +1010,7 @@ sub set_archlinux_service_status
   }
 
   &Utils::Replace::set_sh ($rcconf, "DAEMONS", $daemons);
+  &run_rcng_script ($service, ($active) ? "start" : "stop");
 }
 
 sub set_rcng_services
@@ -930,9 +1031,9 @@ sub set_rcng_services
   foreach $service (@$services)
   {
     $script    = $$service[0];
-    $runlevels = $$service[2];
+    $runlevels = $$service[1];
     $runlevel  = $$runlevels[0];
-    $action    = ($$runlevel[1] eq "start")? 1 : 0;
+    $action    = ($$runlevel[1] == $SERVICE_START)? 1 : 0;
 
     &$func ($script, $action);
   }
@@ -942,58 +1043,43 @@ sub set_rcng_services
 sub get_suse_service_info ($service)
 {
   my ($service) = @_;
-  my (%hash, @arr, @ret);
+  my (@runlevels, $link, $runlevel);
                                                                                                                                                              
-  # We have to check if the service is in the "forbidden" list
-  return undef if (&Init::ServicesList::is_forbidden ($service));
-                                                                                                                                                             
-  $hash{"script"} = $service;
-
   foreach $link (<$rcd_path/rc[0-9S].d/S[0-9][0-9]$service>)
   {
     $link =~ s/$rcd_path\///;
     $link =~ /rc([0-6])\.d\/S[0-9][0-9].*/;
     $runlevel = $1;
 
-    push @arr, { "name"   => $runlevel,
-                 "action" => "start" };
+    push @runlevels, [ $runlevel, $SERVICE_START, 0 ];
   }
 
   foreach $link (<$rcd_path/boot.d/S[0-9][0-9]$service>)
   {
-    push @arr, {"name"   => "B",
-                "action" => "start" };
+    push @runlevels, [ "B", $SERVICE_START, 0 ];
   }
 
-  if (scalar @arr > 0)
-  {
-    push @ret, { "runlevel" => \@arr };
-    $hash{"runlevels"} = \@ret;
-    $hash{"role"} = &Init::ServicesList::get_role ($service);
-  }
-
-  return \%hash;
+  return ($service, $runlevels);
 }
 
 sub get_suse_services
 {
-  my ($service, %ret);
+  my ($service, @arr);
 
   ($rcd_path, $initd_path) = &get_sysv_paths ();
 
   foreach $service (<$gst_prefix/etc/init.d/*>)
   {
-    my (%hash);
+    my (@info);
 
     next if (-d $service || ! -x $service);
 
     $service =~ s/.*\///;
-    $hash = &get_suse_service_info ($service);
-
-    $ret{$service} = $hash if ($hash ne undef);
+    @info = &get_suse_service_info ($service);
+    push @arr, \@info  if (scalar (@info));
   }
 
-  return \%ret;
+  return \@arr;
 }
 
 # This function stores the configuration in suse init
@@ -1001,21 +1087,29 @@ sub set_suse_services
 {
   my ($services) = @_;
   my ($action, $runlevels, $script, $rllist);
+  my (%configured_runlevels, $default_runlevel);
+
+  $default_runlevel = &get_sysv_default_runlevel ();
 
   foreach $service (@$services)
   {
     $script = $$service[0];
-    $runlevels = $$service[2];
+    $runlevels = $$service[1];
     $rllist = "";
+    %configured_runlevels = {};
 
     &Utils::File::run ("insserv -r $script");
 
     foreach $rl (@$runlevels)
     {
-      if ($$rl[1] eq "start")
+      $configured_runlevels{$$rl[0]} = 1;
+
+      if ($$rl[1] == $SERVICE_START)
       {
         $rllist .= $$rl[0] . ",";
       }
+
+      &run_sysv_initd_script ($script, ($$rl[1] == $SERVICE_START) ? "start" : "stop");
     }
 
     if ($rllist ne "")
@@ -1024,6 +1118,11 @@ sub set_suse_services
 
       &Utils::File::run ("insserv $script,start=$rllist");
     }
+
+    if (!$configured_runlevels{$default_runlevel})
+    {
+      &run_sysv_initd_script ($script, $$rl[1]);
+    }
   }
 }
 
@@ -1060,6 +1159,25 @@ sub get_init_type
   }
 }
 
+sub run_script
+{
+  my ($service, $arg) = @_;
+  my ($proc, $type);
+  my %map =
+    (
+     "sysv"    => \&run_sysv_initd_script,
+     "file-rc" => \&run_sysv_initd_script,
+     "bsd"     => \&run_bsd_script,
+     "gentoo"  => \&run_gentoo_script,
+     "rcng"    => \&run_rcng_script,
+     "suse"    => \&run_sysv_initd_script,
+    );
+
+  $type = &get_init_type ();
+  $proc = $map {$type};
+  &$proc ($service, $arg);
+}
+
 sub get
 {
   $type = &get_init_type ();
